
%BZ NOR gate simulation
%by Jacob M. Gold

%Note: takes a couple minutes

%ENTER INPUT 1 HERE (1 for TRUE, 0 for FALSE)
input_1 = 0;

%ENTER INPUT 2 HERE (1 for TRUE, 0 for FALSE)
input_2 = 1;

%That's it!


% initialize cell array of reactions
x_init = initialLeda();
reactions = cell(2, 2);
reactions{1, 1} = @reactorLeda;
reactions{1, 2} = @reactorLeda;
reactions{2, 1} = @reactorLeda;

% no light
light_fns = cell(2, 2);
light_fns{1, 1} = @(t)(0);
if input_1
    light_fns{2, 1} = @(t)(input_light(t, 450, 600, 1e-4));
else
    light_fns{2, 1} = @(t)(1e-4);
end
if input_2
    light_fns{1, 2} = @(t)(input_light(t, 450, 600, 1e-4));
else
    light_fns{1, 2} = @(t)(1e-4);
end

% initialize two drops from interpolated one drop data, couple
% for delta t
x_duo_init = [];
x_duo_init(1, 1, :) = x_init;
x_duo_init(1, 2, :) = x_init;
x_duo_init(2, 1, :) = x_init;

%solve the ODEs
options = odeset('NonNegative', 1, 'RelTol',1e-7,'AbsTol',1e-7);
[t_duo, x_duo] = ode45(@coupled_drop_ODE, [0, 800], x_duo_init, options, [1, 1; 1, 0], parametersLeda, reactions, light_fns);
x_duo = reshape(x_duo, [numel(t_duo), 4, numel(x_init)]);

%check if output spike falls in reading frame
[peaks, locs] = findpeaks(squeeze(x_duo(:, 1, 3)), 'MinPeakHeight', 2.5e-3);
if any((t_duo(locs) > 520) & (t_duo(locs) < 570))
    out = 'TRUE';
else
    out = 'FALSE';
end

%plot stuff
f = figure();

set(f, 'Units', 'Normalized', 'Position', [.1, .1, .7, .55])

plot_color = [1, 0, 1];

if input_1
    in1 = 'TRUE';
else
    in1 = 'FALSE';
end
if input_2
    in2 = 'TRUE';
else
    in2 = 'FALSE';
end

subplot(3, 1, 1);
hold on
plot(t_duo, x_duo(:, 2, 3) * 1000, 'Color', plot_color, 'LineWidth', 4)
box on
title_string = strcat(in1, '$\otimes$  ', in2, ' $\mapsto$ ', out);
title(title_string, 'Interpreter', 'LaTeX', 'FontSize', 26)
axis([0, 800, 0, 3])
ylabel('In 1', 'Interpreter', 'LaTeX', 'FontSize', 26)
set(gca, 'FontSize', 20, 'YTick', [0, 3])

subplot(3, 1, 2);
hold on
rectangle('Position', [520, 0, 50, 3], 'EdgeColor', 'None', 'FaceColor', [.3, .8, .3]);
plot(t_duo, x_duo(:, 1, 3) * 1000, 'Color', plot_color, 'LineWidth', 4)
box on
axis([0, 800, 0, 3])
ylabel('Out', 'Interpreter', 'LaTeX', 'FontSize', 26)
set(gca, 'FontSize', 20, 'YTick', [0, 3])

text(510, 1.5, 'Reading Frame', 'horizontalAlignment', 'right', 'verticalAlignment', 'middle', 'FontSize', 20, 'Interpreter', 'LaTeX')
text(-90, 1.5, 'Ferriin (mM)', 'horizontalAlignment', 'center', 'verticalAlignment', 'middle', 'FontSize', 24, 'Rotation', 90, 'Interpreter', 'LaTeX');

subplot(3, 1, 3);
hold on
plot(t_duo, x_duo(:, 3, 3) * 1000, 'Color', plot_color, 'LineWidth', 4)
box on
axis([0, 800, 0, 3])
ylabel('In 2', 'Interpreter', 'LaTeX', 'FontSize', 26)
xlabel('Time (s)', 'Interpreter', 'LaTeX', 'FontSize', 26)
set(gca, 'FontSize', 20, 'YTick', [0, 3])


